/* 
-*- coding: latin-1 -*-

This file is part of RefactorErl.

RefactorErl is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published
by the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

RefactorErl is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with RefactorErl.  If not, see <http://plc.inf.elte.hu/erlang/>.

The Original Code is RefactorErl.

The Initial Developer of the Original Code is Eötvös Loránd University.
Portions created  by Eötvös Loránd University and ELTE-Soft Ltd.
are Copyright 2007-2024 Eötvös Loránd University, ELTE-Soft Ltd.
and Ericsson Hungary. All Rights Reserved.
*/

package com.refactorerl.ui.presentation.dupcode;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.handlers.HandlerUtil;

import com.refactorerl.ui.logic.dupcode.GroupData;
import com.refactorerl.ui.logic.refactoring.RefactoringRequestFactory;
import com.refactorerl.ui.presentation.Activator;
import com.refactorerl.ui.presentation.Activator.ErrorContainer;

public class ElimDupGroupHandler extends AbstractHandler {

	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {

		IWorkbenchWindow activeWorkbenchWindow = HandlerUtil
				.getActiveWorkbenchWindow(event);
		IWorkbenchPage activePage = activeWorkbenchWindow.getActivePage();

		ClonesResultView view;
		view = (ClonesResultView) activePage
				.findView("com.refactorerl.ui.cloneResultsView"); //$NON-NLS-1$

		GroupData group = view.getSelection();
		if (group == null)
			return null;
		int gid = group.getGroupId();

		String cloneName = view.getName();

		String functionName = showInputDialog(activeWorkbenchWindow,
				"Extract Function", "Specify a name for the new function",
				"extracted_function");

		ErrorContainer err = new ErrorContainer();
		Activator.executeRequest(RefactoringRequestFactory
				.createEliminateDupcodeRequest(cloneName, gid, functionName,
						Activator.getProxy(), null), activeWorkbenchWindow
				.getShell(), false, err);

		if (!err.hasError) {

			MessageDialog.openInformation(
					activeWorkbenchWindow.getShell(),
					"Clone elimination successful",
					"Duplicates were extracted to function '" + functionName + "'.\n"
							+ "Modifications occured on the following positions: \n"
							+ group.getMembers());
		//	DupCodeHelper.closeCloneEditors(view.getInput());
			DupCodeHelper.closeCloneEditors();
			DupCodeHelper.closeCloneResultsView();	
		}

		return null;
	}
	
	private String showInputDialog(IWorkbenchWindow wb, String dialogTitle,
			String dialogMessage, String defaultInput) {
		InputDialog dialog = new InputDialog(wb.getShell(), dialogTitle,
				dialogMessage, defaultInput, null);
		dialog.open();
		String newName = dialog.getValue();
		return newName;
	}
}
